{% extends "admin/delete_confirmation.html" %}
{% load i18n admin_urls %}

{% block content %}
{{block.super}}
{% if not perms lacking and not protected %}
    <form action="" method="post">{% csrf_token %}
    <div>
    <input type="hidden" name="post" value="yes" />
    <input type="password" name="password" >
    <input type="submit" value="{% trans "Yes, I'm sure" %}" />
    </div>
    </form>
{% endif %}
{% endblock %}
